<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All right's reserved.
 *
 * Author: Carmine D'Amore <carmine@pinpointdesigns.co.uk>
 * Code Release Version: 1.0
 * Website: http://www.pinpointdesigns.co.uk
 *
 * Class FinancialCalculator
 * @package Pinpoint\V12Finance\Block
 */

namespace Pinpoint\V12Finance\Block;

use Magento\Catalog\Model\Product;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Registry;
use Magento\Framework\View\Element\Template;
use Magento\Catalog\Model\CategoryFactory;
use Magento\Catalog\Block\Product\ListProduct;
use Magento\Framework\Pricing\Helper\Data as PriceHelper;
use Pinpoint\V12Finance\Helper\FinancialCalculatorData;

class FinancialCalculator extends Template
{
    const MIN_PRICE_REQUIRED = 350.00;
    const INTEREST_FEE = 9.9;

    /**
     * @var Registry
     */
    protected $registry;

    /**
     * @var Product
     */
    private $product;


    /**
     * @var FinancialCalculatorData
     */
    protected $_helperData;

    /**
     * @var CategoryFactory
     */
    protected $_categoryFactory;

    /**
     * @var PriceHelper
     */
    protected $_priceHelper;

    /**
     * @var ListProduct
     */
    protected $_listProduct;

    public function __construct(
        Template\Context $context,
        CategoryFactory $categoryFactory,
        Registry $registry,
        array $data,
        FinancialCalculatorData $helperData,
        PriceHelper $pricehelper,
        ListProduct $listProduct
    )
    {
        $this->registry = $registry;
        $this->_categoryFactory = $categoryFactory;
        $this->_helperData = $helperData;
        $this->_priceHelper = $pricehelper;
        $this->_listProduct = $listProduct;
        parent::__construct($context, $data);
    }

    /**
     * @return Product
     */
    public function getProduct()
    {
        if (is_null($this->product)) {
            $this->product = $this->registry->registry('product');

            if (!$this->product->getId()) {
                throw new LocalizedException(__('Failed to initialize product'));
            }
        }
        return $this->product;
    }

    /**
     * @return ListProduct
     */
    public function getListProduct()
    {
        return $this->_listProduct;
    }

    /**
     * @return string
     */
    public function getProductPrice()
    {
        $price =  $this->getProduct()->getPrice();
        return $this->_priceHelper->currency($price, true, false);
    }

    /**
     * @return float
     */
    public function getPrice()
    {
        $price =  $this->getProduct()->getPrice();
        return $this->_priceHelper->currency($price, false, false);
    }

    /**
     * @return string
     */
    public function getProductMonthlyPrice()
    {
        $price = $this->getProduct()->getPrice();
        if ($price < self::MIN_PRICE_REQUIRED) {
            return null;
        }
        $defaultMonth = $this->_helperData->getDefaultMonthOptionToArray();
        $defaultDeposit =  $this->_helperData->getDefaultDepositOption();
        $monthlyPrice = $this->_helperData->calculateMonthlyPrice($price, $defaultMonth, $defaultDeposit);
        return $this->_priceHelper->currency($monthlyPrice, true, false);
    }

    /**
     * @return array
     */
    public function getMonthOptions()
    {
        return $this->_helperData->getMonthsRangeList();
    }

    /**
     * @return string
     */
    public function getDefaultMonthOption()
    {
        return $this->_helperData->getDefaultMonthOption();
    }

    /**
     * @return int
     */
    public function getNumberOfPayments()
    {
        /* @var array $defaultMonthOption */
        $defaultMonthOption = $this->_helperData->getDefaultMonthOptionToArray();

        /* @var int $numberOfPayments */
        $numberOfPayments = (int) $defaultMonthOption[0];

        return $numberOfPayments;
    }

    /**
     * @return array
     */
    public function getDepositOptions()
    {
        return $this->_helperData->getDepositRangeList();
    }

    /**
     * @return int
     */
    public function getDefaultDepositOption()
    {
        return (int) $this->_helperData->getDefaultDepositOption();
    }

    /**
     * @return string
     */
    public function getDefaultDepositToPay()
    {
        $price = $this->_helperData->getPriceForCalculation($this->getProduct()->getPrice());
        $defaultDeposit = $this->_helperData->getDefaultDepositOption();
        $depositToPay =  $this->_helperData->calculateDepositToPay($price, $defaultDeposit);
        return $this->_priceHelper->currency($depositToPay, true, false);
    }

    /**
     * @return string
     */
    public function getDefaultLoanAmount()
    {
        $price = $this->_helperData->getPriceForCalculation($this->getProduct()->getPrice());
        $deposit = $this->_helperData->calculateDefaultDepositToPay($price);
        $loanAmount =  $this->_helperData->calculateLoanAmount($price, $deposit);
        return $this->_priceHelper->currency($loanAmount, true, false);
    }

    /**
     * @return string
     */
    public function getDefaultLoanRepayment()
    {
        $price = $this->_helperData->getPriceForCalculation($this->getProduct()->getPrice());
        $deposit = $this->_helperData->calculateDefaultDepositToPay($price);
        $loanAmount = $this->_helperData->calculateLoanAmount($price, $deposit);
        $defaultMonth = $this->_helperData->getDefaultMonthOptionToArray();
        $defaultDeposit = $this->_helperData->getDefaultDepositOption();
        $costOfLoan =$this->_helperData->calculateCostOfLoan($price, $deposit, $defaultMonth, $defaultDeposit);
        $loanRepayment=  $this->_helperData->calculateRepayment($loanAmount, $costOfLoan);
        return $this->_priceHelper->currency($loanRepayment, true, false);
    }

    /**
     * @return string
     */
    public function getDefaultCostOfLoan()
    {
        $price = $this->_helperData->getPriceForCalculation($this->getProduct()->getPrice());
        $deposit = $this->_helperData->calculateDefaultDepositToPay($price);
        $defaultMonth = $this->_helperData->getDefaultMonthOptionToArray();
        $defaultDeposit = $this->_helperData->getDefaultDepositOption();
        $costOfLoan =  $this->_helperData->calculateCostOfLoan($price, $deposit, $defaultMonth, $defaultDeposit);
        return $this->_priceHelper->currency($costOfLoan, true, false);

    }

    /**
     * @return string
     */
    public function getTotalAmountPayable()
    {
        $price = $this->_helperData->getPriceForCalculation($this->getProduct()->getPrice());
        $deposit = $this->_helperData->calculateDefaultDepositToPay($price);
        $loanAmount = $this->_helperData->calculateLoanAmount($price, $deposit);
        $defaultMonth = $this->_helperData->getDefaultMonthOptionToArray();
        $defaultDeposit = $this->_helperData->getDefaultDepositOption();
        $costOfLoan =$this->_helperData->calculateCostOfLoan($price, $deposit, $defaultMonth, $defaultDeposit);
        $loanRepayment=  $this->_helperData->calculateRepayment($loanAmount, $costOfLoan);
        $totalAmountPayable =  $this->_helperData->calculateAmountPayable($deposit, $loanRepayment);
        return $this->_priceHelper->currency($totalAmountPayable, true, false);
    }

    /**
     * @return string
     */
    public function getAjaxUrl()
    {
        return $this->getUrl("v12finance/calculator/calculate");
    }
}
