<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All right's reserved.
 *
 * Author: Carmine D'Amore <carmine@pinpointdesigns.co.uk>
 * Code Release Version: 1.0
 * Website: http://www.pinpointdesigns.co.uk
 *
 * Class FinancialCalculatorData
 * @package Pinpoint\V12Finance\Helper
 */

namespace Pinpoint\V12Finance\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Encryption\EncryptorInterface;
use Magento\Framework\Pricing\Helper\Data as PriceHelper;
use Pinpoint\V12Finance\Model\Adminhtml\System\Config\Source\MonthsOption;
use Pinpoint\V12Finance\Model\Adminhtml\System\Config\Source\DepositOption;
use Pinpoint\V12Finance\Model\Adminhtml\System\Config\Source\InterestOption;

class FinancialCalculatorData extends AbstractHelper
{
    const MIN_PRICE_REQUIRED = 350.00;
    const INTEREST_FEE = 9.9;

    /**
     * @var EncryptorInterface
     */
    protected $encryptor;

    /**
     * @var MonthsOption
     */
    protected $_monthOptions;

    /**
     * @var DepositOption
     */
    protected $_depositOptions;

    /**
     * @var InterestOption
     */
    protected $_interestOptions;

    /**
     * @var PriceHelper
     */
    protected $_priceHelper;

    /**
     * @param Context $context
     * @param EncryptorInterface $encryptor
     * @param MonthsOption $monthsOption
     * @param DepositOption $depositOption
     * @param PriceHelper $priceHelper
     */
    public function __construct(
        Context $context,
        EncryptorInterface $encryptor,
        MonthsOption $monthsOption,
        DepositOption $depositOption,
        PriceHelper $priceHelper
    )
    {
        $this->encryptor = $encryptor;
        $this->_monthOptions = $monthsOption;
        $this->_depositOptions = $depositOption;
        $this->_priceHelper = $priceHelper;
        parent::__construct($context);
    }

    /*
     * @return string
     */
    public function getDefaultDepositOption($scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT)
    {
        return $this->scopeConfig->getValue(
            'financialcalculator/general/default_deposit',
            $scope
        );
    }

    /*
     * @return string
     */
    public function getDefaultMonthOption($scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT)
    {
        return $this->scopeConfig->getValue(
            'financialcalculator/general/default_month',
            $scope
        );
    }


    /*
     * Convert the default config Month Option string in array
     * @return array
     */
    public function getDefaultMonthOptionToArray($scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT)
    {
        /* @var array $defaultMonthOption */
        $defaultMonthOption = explode("_", $this->scopeConfig->getValue(
            'financialcalculator/general/default_month',
            $scope
        ));

        return $defaultMonthOption;
    }

    /*
     * @return array
     */
    public function getMonthsRangeList($scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT)
    {
        $list = $this->scopeConfig->getValue("financialcalculator/general/months_range", $scope);
        $list = explode(',', $list);
        $monthOptions = $this->_monthOptions->getOriginalOption();

        foreach ($monthOptions as $key => $value) {
            if (in_array($key, $list)) {
                $result[$key] = $value;
            }
        }
        return $result;
    }

    /*
     * @return array
     */
    public function getDepositRangeList($scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT)
    {
        $list = $this->scopeConfig->getValue("financialcalculator/general/deposit_range", $scope);
        $list = explode(',', $list);
        $depositOptions = $this->_depositOptions->getOriginalOption();
        $result = array();
        foreach ($depositOptions as $key => $value) {
            if (in_array($key, $list)) {
                $result[$key] = $value;
            }
        }
        return $result;
    }

    /*
     * @return float
     */
    public function getPriceForCalculation($price, $format = false)
    {
        return $this->_priceHelper->currency($price, $format, false);
    }

    /*
     * @return float
     */
    public function calculateDefaultDepositToPay($price, $format = false)
    {
        $dep = (int) $this->getDefaultDepositOption();
        $deposit = ($price / 100) * $dep;
        return $this->_priceHelper->currency($deposit, $format, false);
    }

    /*
     * @return float
     */
    public function calculateDepositToPay($price, $dep, $format = false)
    {
        $deposit = ($price / 100) * $dep;
        return $this->_priceHelper->currency($deposit, $format, false);
    }

    /*
     * @return string
     */
    public function printDepositToPay ($price, $dep, $format = true)
    {
        return $this->calculateDepositToPay($price, $dep, $format);
    }

    /*
     * @return float
     */
    public function calculateLoanAmount($price, $deposit, $format = false)
    {
        $loanAmount = $price - $deposit;
        return $this->_priceHelper->currency($loanAmount, $format, false);
    }

    /*
     * @return float
     */
    public function calculateRepayment($loanAmount, $costOfLoan, $format = false)
    {
        $loanReplayment = $loanAmount + $costOfLoan;
        return $this->_priceHelper->currency($loanReplayment, $format, false);
    }

    /*
     * @return float
     */
    public function calculateCostOfLoan($price,$deposit,array $selectedMonthOptions, $format = false)
    {
        $loanReplayment = $price - $deposit;
        $selectedMonth = (int) $selectedMonthOptions[0];

        $apr = (float) $selectedMonthOptions[2];
        $calcFactor = (float)$selectedMonthOptions[1];

        if($apr === 0.00){
            $costOfLoan = 0;
        }
        else{
            $monthlyPayment = $loanReplayment * $calcFactor;
            $monthlyPayment = round($monthlyPayment*100)/100;
            $costOfLoan = $monthlyPayment * $selectedMonth;
            $costOfLoan = $costOfLoan - $loanReplayment;
        }

        return $this->_priceHelper->currency($costOfLoan, false, false);
    }

    /*
     * @return float
     */
    public function calculateAmountPayable($deposit, $loanReplayment, $format = false)
    {
        $totalAmountPayable= $deposit + $loanReplayment;
        return $this->_priceHelper->currency($totalAmountPayable, $format, false);
    }

    /*
     * @param float $price
     * @param array $selectedMonthOptions
     * @param int $selectedDeposit
     * @param bolean $format
     *
     * return float
     */
    public function calculateMonthlyPrice($price, array $selectedMonthOptions, $selectedDeposit, $format = false)
    {

        if ($price < self::MIN_PRICE_REQUIRED) {
            return null;
        }

        $selectedMonth = (int) $selectedMonthOptions[0];

        $deposit = ($price / 100) * $selectedDeposit;

        $loanReplayment = $price - $deposit;

        $apr = (float) $selectedMonthOptions[2];
        $calcFactor = (float)$selectedMonthOptions[1];

        if($apr === 0.00){
            $monthlyPayment = $loanReplayment/$selectedMonth;
        }
        else{
            $monthlyPayment = $loanReplayment * $calcFactor;
        }

        $monthlyPayment = round($monthlyPayment*100)/100;

        return $this->_priceHelper->currency($monthlyPayment, $format, false);
    }

    /*
     * @return string
     */
    public function getPriceFormatted($price)
    {
        return $this->_priceHelper->currency($price, true, false);
    }

}
